searchData={"items":[{"type":"module","title":"ShortUUID","doc":"ShortUUID is a module for encoding and decoding UUIDs using a base57 alphabet.","ref":"ShortUUID.html"},{"type":"module","title":"Functions - ShortUUID","doc":"- `encode/1` - Encodes a UUID into a shorter string using base57 alphabet\n- `encode!/1` - Same as encode/1 but raises an error on failure\n- `decode/1` - Decodes a shortened string back into a UUID\n- `decode!/1` - Same as decode/1 but raises an error on failure","ref":"ShortUUID.html#module-functions"},{"type":"module","title":"Example - ShortUUID","doc":"iex> ShortUUID.encode(\"550e8400-e29b-41d4-a716-446655440000\")\n    {:ok, \"H9cNmGXLEc8NWcZzSThA9S\"}\n\n    iex> ShortUUID.decode(\"H9cNmGXLEc8NWcZzSThA9S\")\n    {:ok, \"550e8400-e29b-41d4-a716-446655440000\"}\n\nFor custom alphabets and more options, see `ShortUUID.Builder`.\nTo implement your own compatible ShortUUID module, use `ShortUUID.Behaviour`.","ref":"ShortUUID.html#module-example"},{"type":"function","title":"ShortUUID.decode/1","doc":"","ref":"ShortUUID.html#decode/1"},{"type":"function","title":"ShortUUID.decode!/1","doc":"","ref":"ShortUUID.html#decode!/1"},{"type":"function","title":"ShortUUID.encode/1","doc":"","ref":"ShortUUID.html#encode/1"},{"type":"function","title":"ShortUUID.encode!/1","doc":"","ref":"ShortUUID.html#encode!/1"},{"type":"behaviour","title":"ShortUUID.Behaviour","doc":"Defines the behavior for ShortUUID-compatible modules.\n\nThis behavior ensures consistent interface across different ShortUUID implementations.\nAny module implementing this behavior should provide encode/decode functionality\nfor converting UUIDs to shorter string representations and back.","ref":"ShortUUID.Behaviour.html"},{"type":"behaviour","title":"Required Callbacks - ShortUUID.Behaviour","doc":"- `encode/1` - Encodes a standard UUID into a shorter string\n- `encode!/1` - Encodes a UUID, raising an exception on invalid input\n- `decode/1` - Decodes a shortened UUID string back into standard UUID format\n- `decode!/1` - Decodes a shortened UUID, raising an exception on invalid input","ref":"ShortUUID.Behaviour.html#module-required-callbacks"},{"type":"callback","title":"ShortUUID.Behaviour.decode/1","doc":"Decodes a shortened UUID string back into standard UUID format.","ref":"ShortUUID.Behaviour.html#c:decode/1"},{"type":"callback","title":"Parameters - ShortUUID.Behaviour.decode/1","doc":"- `string` - A shortened UUID string","ref":"ShortUUID.Behaviour.html#c:decode/1-parameters"},{"type":"callback","title":"Returns - ShortUUID.Behaviour.decode/1","doc":"- `{:ok, uuid}` - Successfully decoded UUID\n- `{:error, message}` - Error with descriptive message","ref":"ShortUUID.Behaviour.html#c:decode/1-returns"},{"type":"callback","title":"ShortUUID.Behaviour.decode!/1","doc":"Decodes a shortened UUID string back into standard UUID format.\nRaises an ArgumentError if the input is invalid.","ref":"ShortUUID.Behaviour.html#c:decode!/1"},{"type":"callback","title":"Parameters - ShortUUID.Behaviour.decode!/1","doc":"- `string` - A shortened UUID string","ref":"ShortUUID.Behaviour.html#c:decode!/1-parameters"},{"type":"callback","title":"Returns - ShortUUID.Behaviour.decode!/1","doc":"- `uuid` - Successfully decoded UUID","ref":"ShortUUID.Behaviour.html#c:decode!/1-returns"},{"type":"callback","title":"Raises - ShortUUID.Behaviour.decode!/1","doc":"- `ArgumentError` - If the input is invalid","ref":"ShortUUID.Behaviour.html#c:decode!/1-raises"},{"type":"callback","title":"ShortUUID.Behaviour.encode/1","doc":"Encodes a UUID string into a shorter string representation.","ref":"ShortUUID.Behaviour.html#c:encode/1"},{"type":"callback","title":"Parameters - ShortUUID.Behaviour.encode/1","doc":"- `uuid` - A standard UUID string (with or without hyphens)","ref":"ShortUUID.Behaviour.html#c:encode/1-parameters"},{"type":"callback","title":"Returns - ShortUUID.Behaviour.encode/1","doc":"- `{:ok, encoded}` - Successfully encoded string\n- `{:error, message}` - Error with descriptive message","ref":"ShortUUID.Behaviour.html#c:encode/1-returns"},{"type":"callback","title":"ShortUUID.Behaviour.encode!/1","doc":"Encodes a UUID string into a shorter string representation.\nRaises an ArgumentError if the input is invalid.","ref":"ShortUUID.Behaviour.html#c:encode!/1"},{"type":"callback","title":"Parameters - ShortUUID.Behaviour.encode!/1","doc":"- `uuid` - A standard UUID string (with or without hyphens)","ref":"ShortUUID.Behaviour.html#c:encode!/1-parameters"},{"type":"callback","title":"Returns - ShortUUID.Behaviour.encode!/1","doc":"- `encoded` - Successfully encoded string","ref":"ShortUUID.Behaviour.html#c:encode!/1-returns"},{"type":"callback","title":"Raises - ShortUUID.Behaviour.encode!/1","doc":"- `ArgumentError` - If the input is invalid","ref":"ShortUUID.Behaviour.html#c:encode!/1-raises"},{"type":"module","title":"ShortUUID.Builder","doc":"ShortUUID.Builder is a module for encoding and decoding UUIDs (Universally Unique Identifiers) using various predefined or custom alphabets.","ref":"ShortUUID.Builder.html"},{"type":"module","title":"Usage - ShortUUID.Builder","doc":"To create your module, simply `use` it and optionally provide an alphabet option:\n\n    defmodule MyModule do\n      use ShortUUID.Builder, alphabet: :base58\n    end\n\nThe `alphabet` option must be one of the predefined alphabets or a custom string (16+ unique characters).","ref":"ShortUUID.Builder.html#module-usage"},{"type":"module","title":"Predefined Alphabets - ShortUUID.Builder","doc":"The following predefined alphabets are available:\n\n- `:base57_shortuuid` - \"23456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\"\n- `:base32` - \"ABCDEFGHIJKLMNOPQRSTUVWXYZ234567\"\n- `:base32_crockford` - \"0123456789ABCDEFGHJKMNPQRSTVWXYZ\"\n- `:base32_hex` - \"0123456789ABCDEFGHIJKLMNOPQRSTUV\"\n- `:base32_z` - \"ybndrfg8ejkmcpqxot1uwisza345h769\"\n- `:base58` - \"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\"\n- `:base62` - \"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz\"\n- `:base64` - \"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/\"\n- `:base64_url` - \"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_\"","ref":"ShortUUID.Builder.html#module-predefined-alphabets"},{"type":"module","title":"Functions - ShortUUID.Builder","doc":"The following functions are available for encoding and decoding UUIDs:\n\n- `encode/1` - Encodes a UUID using the specified or default alphabet.\n- `encode!/1` - Encodes a UUID using the specified or default alphabet, raising an error on failure.\n- `decode/1` - Decodes a string into a UUID using the specified or default alphabet.\n- `decode!/1` - Decodes a string into a UUID using the specified or default alphabet, raising an error on failure.","ref":"ShortUUID.Builder.html#module-functions"},{"type":"module","title":"Example - ShortUUID.Builder","doc":"iex> ShortUUID.encode(\"550e8400-e29b-41d4-a716-446655440000\")\n    {:ok, \"H9cNmGXLEc8NWcZzSThA9S\"}\n\n    iex> ShortUUID.decode(\"H9cNmGXLEc8NWcZzSThA9S\")\n    {:ok, \"550e8400-e29b-41d4-a716-446655440000\"}","ref":"ShortUUID.Builder.html#module-example"},{"type":"module","title":"ShortUUID.Core","doc":"Core module for ShortUUID encoding and decoding.\n\nThis module provides the core functionality for encoding and decoding UUIDs\nusing various alphabets. It includes functions for parsing UUIDs, encoding\nthem into shorter strings, and decoding those strings back into UUIDs.","ref":"ShortUUID.Core.html"},{"type":"module","title":"Functions - ShortUUID.Core","doc":"- `parse_uuid/1` - Parses a UUID string into a normalized format.\n- `encode_binary/5` - Encodes a UUID into a shorter string using the specified alphabet.\n- `decode_string/4` - Decodes a shortened string back into a UUID.\n- `format_uuid/1` - Formats an integer value as a UUID string.\n- `encode_int/5` - Encodes an integer value into a string using the specified alphabet.\n- `decode_to_int/3` - Decodes a string into an integer value using the specified alphabet.\n- `int_to_string/4` - Converts an integer value into a string using the specified alphabet.","ref":"ShortUUID.Core.html#module-functions"},{"type":"function","title":"ShortUUID.Core.decode_string/4","doc":"Decodes a shortened string back into a UUID using the specified base and alphabet.\nValidates the input length and ensures the decoded value is within valid UUID range.","ref":"ShortUUID.Core.html#decode_string/4"},{"type":"function","title":"Examples - ShortUUID.Core.decode_string/4","doc":"iex> alphabet = String.graphemes(\"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\") |> Enum.with_index() |> Map.new()\n    iex> ShortUUID.Core.decode_string(\"BWBeN28Vb7cMEx7Ym8AUzs\", 58, alphabet, 22)\n    {:ok, \"550e8400-e29b-41d4-a716-446655440000\"}\n\n    iex> alphabet = String.graphemes(\"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\") |> Enum.with_index() |> Map.new()\n    iex> ShortUUID.Core.decode_string(\"invalid\", 58, alphabet, 22)\n    {:error, \"Invalid input\"}","ref":"ShortUUID.Core.html#decode_string/4-examples"},{"type":"function","title":"ShortUUID.Core.encode_binary/5","doc":"Encodes a UUID string into a shorter string using the specified alphabet and base.\nTakes a UUID string, base number, alphabet tuple, desired length, and padding character.","ref":"ShortUUID.Core.html#encode_binary/5"},{"type":"function","title":"Examples - ShortUUID.Core.encode_binary/5","doc":"iex> alphabet = String.graphemes(\"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\") |> List.to_tuple()\n    iex> ShortUUID.Core.encode_binary(\"550e8400-e29b-41d4-a716-446655440000\", 58, alphabet, 22, \"1\")\n    {:ok, \"BWBeN28Vb7cMEx7Ym8AUzs\"}\n\n    iex> alphabet = String.graphemes(\"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\") |> List.to_tuple()\n    iex> ShortUUID.Core.encode_binary(\"invalid\", 58, alphabet, 22, \"1\")\n    {:error, \"Invalid UUID\"}","ref":"ShortUUID.Core.html#encode_binary/5-examples"},{"type":"function","title":"ShortUUID.Core.encode_int/5","doc":"Encodes a 128-bit integer into a string using the specified base and alphabet.\nPads the result to the desired length using the padding character.","ref":"ShortUUID.Core.html#encode_int/5"},{"type":"function","title":"ShortUUID.Core.format_uuid/1","doc":"Formats a 128-bit integer as a standard UUID string with dashes.\nConverts the integer to a 32-character hex string and inserts dashes in the correct positions.","ref":"ShortUUID.Core.html#format_uuid/1"},{"type":"function","title":"Examples - ShortUUID.Core.format_uuid/1","doc":"iex> ShortUUID.Core.format_uuid(0x550e8400e29b41d4a716446655440000)\n    \"550e8400-e29b-41d4-a716-446655440000\"","ref":"ShortUUID.Core.html#format_uuid/1-examples"},{"type":"type","title":"ShortUUID.Core.normalized_uuid/0","doc":"","ref":"ShortUUID.Core.html#t:normalized_uuid/0"},{"type":"function","title":"ShortUUID.Core.parse_uuid/1","doc":"Parses and normalizes various UUID string formats.","ref":"ShortUUID.Core.html#parse_uuid/1"},{"type":"function","title":"Examples - ShortUUID.Core.parse_uuid/1","doc":"iex> ShortUUID.Core.parse_uuid(\"550e8400-e29b-41d4-a716-446655440000\")\n    {:ok, \"550e8400e29b41d4a716446655440000\"}\n\n    iex> ShortUUID.Core.parse_uuid(\"{550e8400-e29b-41d4-a716-446655440000}\")\n    {:ok, \"550e8400e29b41d4a716446655440000\"}\n\n    iex> ShortUUID.Core.parse_uuid(\"not-a-uuid\")\n    {:error, \"Invalid UUID\"}","ref":"ShortUUID.Core.html#parse_uuid/1-examples"},{"type":"type","title":"ShortUUID.Core.short_uuid/0","doc":"","ref":"ShortUUID.Core.html#t:short_uuid/0"},{"type":"type","title":"ShortUUID.Core.uuid_string/0","doc":"","ref":"ShortUUID.Core.html#t:uuid_string/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v4.0.0 (25.12.2024) - Changelog","doc":"Breaking changes:\n* Dropped support for binary UUID input","ref":"changelog.html#v4-0-0-25-12-2024"},{"type":"extras","title":"Added - Changelog","doc":"* Support for custom alphabets\n* Predefined alphabets (base32, base58, base62, base64, etc.)\n* `ShortUUID.Builder` module for creating custom ShortUUID modules","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* Moved core functionality to `ShortUUID.Core`\n* Simplified main `ShortUUID` module interface\n* Improved error messages and validation\n\n```elixir\n# Old v3.x code still works, the ShortUUID module uses the same alphabet as in V3\n# If you just want to keep using ShortUUID no changes are required\nShortUUID.encode(uuid)\n\n# New in v4.x you can define use one of a list of predefined alphabets or define your own\ndefmodule MyUUID do\n  use ShortUUID.Builder, alphabet: :base58\nend\n\nMyUUID.encode(uuid)\n\ndefmodule MyCustomUUID do\n  use ShortUUID.Builder, alphabet: \"ABCDEFGHIJKLMNOPQRSTUVWXYZ234567\"\nend\n\nMyCustomUUID.encode(uuid)\n```\n\n## [Released]","ref":"changelog.html#changed"},{"type":"extras","title":"v3.0.0 (15.07.2023) - Changelog","doc":"Breaking change, ShortUUIDs created by `  \"UD6ibhr3V4YXvriP822222\" |> String.reverse() |> ShortUUID.decode!()\n\"00000001-0001-0001-0001-000000000001\"\n```","ref":"changelog.html#v3-0-0-15-07-2023"},{"type":"extras","title":"Changed - Changelog","doc":"* move most significant bit to the beginning of the encoded result similar to libraries in other languages (most importantly python shortuuid)\n* drop support for decoding of un-padded ShortUUIDs\n* drop support for formats other than regular hyphenated and unhyphenated UUIDs, MS format and binary UUIDs like are stored in PostgreSQL uuid type\n* refactor code for better readability\n* improve encode and decode performance","ref":"changelog.html#changed"},{"type":"extras","title":"Benchmarks - Changelog","doc":"Results are not comparable to previous benchmarks due to them being run on a different system\n  ```\n  Operating System: macOS\n  CPU Information: Apple M2 Max\n  Number of Available Cores: 12\n  Available memory: 32 GB\n  Elixir 1.15.2\n  Erlang 25.3.2.3\n\n  Benchmark suite executing with the following configuration:\n  warmup: 2 s\n  time: 5 s\n  memory time: 0 ns\n  parallel: 1\n  inputs: none specified\n  ```\n\n* v3.0.0\n  ```\n  Name                                        ips        average  deviation         median         99th %\n  encode/1 binary uuid                  1212.57 K        0.82 μs  ±1995.67%        0.75 μs        1.00 μs\n  encode/1 unhyphenated uuid string      788.79 K        1.27 μs   ±984.70%        1.13 μs        1.63 μs\n  encode/1 hyphenated uuid string        753.56 K        1.33 μs  ±1106.96%        1.17 μs        1.67 μs\n  encode/1 uuid string with braces       722.36 K        1.38 μs  ±1188.51%        1.21 μs        1.83 μs\n  decode/1                                 1.15 M      868.43 ns  ±1506.27%         751 ns        1334 ns\n  ```\n\n* v2.1.1\n  ```\n  Name                                        ips        average  deviation         median         99th %\n  encode/1 binary uuid                    1018.43 K        0.98 μs  ±1224.91%        0.88 μs        1.25 μs\n  encode/1 unhyphenated uuid string        849.67 K        1.18 μs  ±1171.07%        1.08 μs        1.42 μs\n  encode/1 hyphenated uuid string          731.91 K        1.37 μs   ±691.40%        1.29 μs        1.63 μs\n  encode/1 uuid string with braces         569.16 K        1.76 μs   ±833.41%        1.63 μs        2.17 μs\n  decode/1                                   1.00 M      996.37 ns   ±781.87%         918 ns        1376 ns\n  ```","ref":"changelog.html#benchmarks"},{"type":"extras","title":"v2.1.1 (2019-02-18) - Changelog","doc":"* speed improvements\n\n  Benchmarked on 2018 Macbook Pro 13 (non-touch), results are just a snapshot\n  and not averaged.\n\n  before:\n\n  ```","ref":"changelog.html#v2-1-1-2019-02-18"},{"type":"extras","title":"ShortUUIDBench - Changelog","doc":"benchmark name                       iterations   average time\n  encode/1 uuid binary                     500000   5.93 µs/op\n  encode/1 uuid string not hyphenated      100000   10.71 µs/op\n  encode/1 uuid string                     100000   15.35 µs/op\n  encode/1 uuid string with braces         100000   17.22 µs/op\n  decode/1                                 100000   15.05 µs/op\n  ```\n\n  after:\n\n  ```","ref":"changelog.html#shortuuidbench"},{"type":"extras","title":"ShortUUIDBench - Changelog","doc":"benchmark name                       iterations   average time\n  encode/1 uuid binary                     500000   3.54 µs/op\n  encode/1 uuid string not hyphenated      500000   4.12 µs/op\n  encode/1 uuid string                     500000   4.54 µs/op\n  encode/1 uuid string with braces         500000   5.84 µs/op\n  decode/1                                 500000   7.97 µs/op\n  ```","ref":"changelog.html#shortuuidbench"},{"type":"extras","title":"v2.1.0 (2019-02-08) - Changelog","doc":"* support directly encoding binary UUID","ref":"changelog.html#v2-1-0-2019-02-08"},{"type":"extras","title":"v2.0.1 (2019-01-31) - Changelog","doc":"* add error fallbacks for encode/decode for the case where input is not string\n* update test cases\n* update docs","ref":"changelog.html#v2-0-1-2019-01-31"},{"type":"extras","title":"v2.0.0 (2019-01-29) - Changelog","doc":"* drop support for custom alphabets\n* use fixed alphabet _23456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz_ as it seems to be by far the most widely used shortuuid alphabet","ref":"changelog.html#v2-0-0-2019-01-29"},{"type":"extras","title":"License","doc":"# The MIT License\n\nCopyright (c) 2024 Goran Pedić\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# ShortUUID\n\n![Build Status](https://github.com/gpedic/ex_shortuuid/actions/workflows/ci.yml/badge.svg?branch=master)\n[![Coverage Status](https://coveralls.io/repos/github/gpedic/ex_shortuuid/badge.svg)](https://coveralls.io/github/gpedic/ex_shortuuid)\n[![Module Version](https://img.shields.io/hexpm/v/shortuuid.svg)](https://hex.pm/packages/shortuuid)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/shortuuid/)\n[![Total Download](https://img.shields.io/hexpm/dt/shortuuid.svg)](https://hex.pm/packages/shortuuid)\n[![License](https://img.shields.io/hexpm/l/shortuuid.svg)](https://github.com/gpedic/ex_shortuuid/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/gpedic/shortuuid.svg)](https://github.com/gpedic/ex_shortuuid/commits/master)\n\n<!-- MDOC !-->\n\nShortUUID is a lightweight Elixir library for generating short, unique IDs in URLs. It turns standard UUIDs into smaller strings ideal for use in URLs.\nYou can choose from a set of predefined alphabets or define your own. \nThe default alphabet includes lowercase letters, uppercase letters, and digits, omitting characters like 'l', '1', 'I', 'O', and '0' to keep them readable.\n\n**Note:** Different ShortUUID implementations be compatible as long as they use the same alphabet. However, there is no official standard, so if you plan to use ShortUUID with other libraries, it's a good idea to research and test for compatibility.\n\nUnlike some other solutions, ShortUUID does not produce UUIDs on its own as there are already plenty of libraries to do so. To generate UUIDs, use libraries such as\n[Elixir UUID](https://github.com/zyro/elixir-uuid), [Erlang UUID](https://github.com/okeuday/uuid) and also [Ecto](https://hexdocs.pm/ecto/Ecto.UUID.html) as it can generate version 4 UUIDs.\n\nShortUUID supports common UUID formats and is case-insensitive.","ref":"readme.html"},{"type":"extras","title":"Compatibility - Overview","doc":"","ref":"readme.html#compatibility"},{"type":"extras","title":"v4.0.0 breaking changes - Overview","doc":"Raw binary UUID input (as `<<...>>`) is no longer supported. UUIDs must be provided as strings in standard UUID format (`\"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx\"`) or as 32-character hex strings without hyphens.\n\nExamples of supported formats:\n```elixir\n# Supported\n\"550e8400-e29b-41d4-a716-446655440000\"  # With hyphens\n\"550e8400e29b41d4a716446655440000\"      # Without hyphens\n\n# No longer supported in v4.0.0\n<<85, 14, 132, 0, 226, 155, 65, 212, 167, 22, 68, 102, 85, 68, 0, 0>>\n```","ref":"readme.html#v4-0-0-breaking-changes"},{"type":"extras","title":"v3.0.0 breaking changes - Overview","doc":"Changed bit order and padding behavior to align with other language implementations:\n- Most significant bits are now encoded first\n- Padding characters appear at the end of the string\n- Compatible with Python's [shortuuid](https://github.com/skorokithakis/shortuuid) v1.0.0+ and Node.js [short-uuid](https://github.com/oculus42/short-uuid)\n\nBefore `v3.0.0`\n```elixir\n\niex> \"00000001-0001-0001-0001-000000000001\" |> ShortUUID.encode\n{:ok, \"UD6ibhr3V4YXvriP822222\"}\n\n```\n\nAfter `v3.0.0`\n```elixir\n\niex> \"00000001-0001-0001-0001-000000000001\" |> ShortUUID.encode\n{:ok, \"222228PirvXY4V3rhbi6DU\"}\n\n```\n\nTo migrate ShortUUIDs created using `  \"UD6ibhr3V4YXvriP822222\" |> String.reverse() |> ShortUUID.decode!()\n\"00000001-0001-0001-0001-000000000001\"\n```\n\n*Warning:* Decoding ShortUUIDs created using a version `  \"UD6ibhr3V4YXvriP822222\" |> ShortUUID.decode!() === \"00000001-0001-0001-0001-000000000001\"\nfalse\niex> \"UD6ibhr3V4YXvriP822222\" |> ShortUUID.decode()\n{:ok, \"933997ef-eb92-293f-b202-2a879fc84be9\"}\n```","ref":"readme.html#v3-0-0-breaking-changes"},{"type":"extras","title":"Installation - Overview","doc":"Add `:shortuuid` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:shortuuid, \"~> 4.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - Overview","doc":"```elixir\niex> \"f98e80e7-9923-4173-8408-98f8254912ad\" |> ShortUUID.encode\n{:ok, \"nQtAWSRQ6ByybDtRs7dQwE\"}\n\niex> \"f98e80e7-9923-4173-8408-98f8254912ad\" |> ShortUUID.encode!\n\"nQtAWSRQ6ByybDtRs7dQwE\"\n\niex> \"nQtAWSRQ6ByybDtRs7dQwE\" |> ShortUUID.decode\n{:ok, \"f98e80e7-9923-4173-8408-98f8254912ad\"}\n\niex> \"nQtAWSRQ6ByybDtRs7dQwE\" |> ShortUUID.decode!\n\"f98e80e7-9923-4173-8408-98f8254912ad\"\n```","ref":"readme.html#examples"},{"type":"extras","title":"Using ShortUUID with Ecto - Overview","doc":"If you would like to use ShortUUID with Ecto schemas try [Ecto.ShortUUID](https://github.com/gpedic/ecto_shortuuid).\n\nIt provides a custom Ecto type which allows for ShortUUID primary and foreign keys while staying compatible with `:binary_key` (`EctoUUID`).","ref":"readme.html#using-shortuuid-with-ecto"},{"type":"extras","title":"Custom Alphabets - Overview","doc":"Starting with version `v4.0.0`, ShortUUID allows you to define custom alphabets for encoding and decoding UUIDs. You can use predefined alphabets or define your own.","ref":"readme.html#custom-alphabets"},{"type":"extras","title":"Restrictions - Overview","doc":"- The alphabet must contain at least 16 unique characters.\n- The alphabet must not contain duplicate characters.","ref":"readme.html#restrictions"},{"type":"extras","title":"Predefined Alphabets - Overview","doc":"Starting with version `v4.0.0`, the following predefined alphabets are available:\n\n- `:base57_shortuuid` - \"23456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\"\n- `:base32` - \"ABCDEFGHIJKLMNOPQRSTUVWXYZ234567\"\n- `:base32_crockford` - \"0123456789ABCDEFGHJKMNPQRSTVWXYZ\"\n- `:base32_hex` - \"0123456789ABCDEFGHIJKLMNOPQRSTUV\"\n- `:base32_z` - \"ybndrfg8ejkmcpqxot1uwisza345h769\"\n- `:base58` - \"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz\"\n- `:base62` - \"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz\"\n- `:base64` - \"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/\"\n- `:base64_url` - \"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_\"","ref":"readme.html#predefined-alphabets"},{"type":"extras","title":"Using a custom or predefined alphabet - Overview","doc":"```elixir\ndefmodule MyBase58UUID do\n  use ShortUUID.Builder, alphabet: :base58\nend\n\ndefmodule MyCustomUUID do\n  use ShortUUID.Builder, alphabet: \"0123456789ABCDEF\"\nend\n\niex> MyBase58UUID.encode(\"550e8400-e29b-41d4-a716-446655440000\")\n{:ok, \"BWBeN28Vb7cMEx7Ym8AUzs\"}\n\niex> MyBase58UUID.decode(\"BWBeN28Vb7cMEx7Ym8AUzs\")\n{:ok, \"550e8400-e29b-41d4-a716-446655440000\"}\n```","ref":"readme.html#using-a-custom-or-predefined-alphabet"},{"type":"extras","title":"Just for fun - Overview","doc":"Since v4.0.0 alphabets are not limited to alphanumeric characters either\n\n```elixir\ndefmodule UnicodeUUID do\n  use ShortUUID.Builder, alphabet: \"🌟💫✨⭐️🌙🌎🌍🌏🌑🌒🌓🌔🌕🌖🌗🌘\"\nend\n\niex> UnicodeUUID.encode(\"550e8400-e29b-41d4-a716-446655440000\")\n{:ok, \"🌎🌎🌟🌗🌑🌙🌟🌟🌗✨🌒🌔🌙💫🌖🌙🌓🌏💫🌍🌙🌙🌍🌍🌎🌎🌙🌙🌟🌟🌟🌟\"}\n\n\ndefmodule SmileyUUID do\n  use ShortUUID.Builder, alphabet: \"😀😊😄😍🥰😘😜🤪😋🤔😌🧐😐😑😶😮😲😱😴🥱😪😢😭😤😎🤓😇😈👻👽🤖🤡💀\"\nend\n\niex> SmileyUUID.encode(\"550e8400-e29b-41d4-a716-446655440000\")\n{:ok, \"😊🤪😢😘💀🥰😲😊🤡🤖🤔😊😘😤👽🤓👻😊👽😲😋😀😭😇😲🤖\"}\n\n```","ref":"readme.html#just-for-fun"},{"type":"extras","title":"Implementing Compatible Modules - Overview","doc":"Starting with version `v4.1.0`, ShortUUID provides a behavior that you can implement to create compatible modules. \nThis is useful if you want to create your own ShortUUID-compatible module with custom functionality while maintaining a consistent interface.\n\n```elixir\ndefmodule MyCustomShortUUID do\n  @behaviour ShortUUID.Behaviour\n\n  @spec encode(String.t()) :: {:ok, String.t()} | {:error, String.t()}\n  def encode(uuid) do\n    # Your custom implementation here\n    {:ok, \"encoded_\" <> uuid}\n  end\n\n  @spec encode!(String.t()) :: String.t() | no_return()\n  def encode!(uuid) do\n    case encode(uuid) do\n      {:ok, encoded} -> encoded\n      {:error, msg} -> raise ArgumentError, message: msg\n    end\n  end\n\n  @spec decode(String.t()) :: {:ok, String.t()} | {:error, String.t()}\n  def decode(encoded) do\n    # Your custom implementation here\n    if String.starts_with?(encoded, \"encoded_\") do\n      {:ok, String.replace_prefix(encoded, \"encoded_\", \"\")}\n    else\n      {:error, \"Invalid format\"}\n    end\n  end\n\n  @spec decode!(String.t()) :: String.t() | no_return()\n  def decode!(encoded) do\n    case decode(encoded) do\n      {:ok, decoded} -> decoded\n      {:error, msg} -> raise ArgumentError, message: msg\n    end\n  end\nend\n```\n\nThe `ShortUUID.Behaviour` requires implementing four callbacks:\n- `encode/1` - Encode a UUID into a shorter representation\n- `encode!/1` - Same as encode/1 but raises on error\n- `decode/1` - Decode a shortened UUID back to standard format\n- `decode!/1` - Same as decode/1 but raises on error\n\nUsing this behavior ensures that your custom implementation will be compatible with code expecting a ShortUUID-compatible module.","ref":"readme.html#implementing-compatible-modules"},{"type":"extras","title":"Documentation - Overview","doc":"Look up the full documentation at [https://hexdocs.pm/shortuuid](https://hexdocs.pm/shortuuid).","ref":"readme.html#documentation"},{"type":"extras","title":"Acknowledgments - Overview","doc":"Inspired by [shortuuid](https://github.com/skorokithakis/shortuuid).","ref":"readme.html#acknowledgments"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2024 Goran Pedić\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. \n\nSee the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.0"}}